import collections

from ..cvssv2 import calculator as v2_calculator
from ..cvssv3 import calculator as v3_calculator


def parse_text_info_score(comment):
    """ Returns the result of parsing a wiki table-like formatted cvss
        score.
    """
    score_d = collections.OrderedDict()
    for line in comment.split("\n"):
        if "||" not in line:
            continue
        info = [item.strip() for item in line.split("|") if item != ""]
        if len(info) < 2:
            continue
        mini_dict = {info[0]: info[1]}
        score_d.update(mini_dict)

    score = None
    version = 3.0
    if len(list(score_d.keys())) == 6:
        version = 2.0
        score, exploit_sub_score = _compute_cvssv2_score(score_d)
    elif len(list(score_d.keys())) == 8:
        score, exploit_sub_score = _compute_cvssv3_score(score_d)
    if score is None:
        return None
    score_d.update({
        'exploitability_sub_score': exploit_sub_score,
        'score': score,
        'version': version,
    })
    return score_d


def _compute_cvssv2_score(comment_dictionary):
    """ Returns the result of computing a cvss v2 score and the
        exploitability subscore from the given comment dictionary.
    """
    impact = v2_calculator.get_impact_score(
        comment_dictionary['ConfImpact'],
        comment_dictionary['IntegImpact'],
        comment_dictionary['AvailImpact'])
    exploit = v2_calculator.get_exploitability_score(
        comment_dictionary['AccessVector'],
        comment_dictionary['AccessComplexity'],
        comment_dictionary['Authentication'])
    score = round(v2_calculator.calc_base_score(impact, exploit), 1)
    return score, round(exploit, 2)


def _compute_cvssv3_score(comment_dictionary):
    """ Returns the result of computing a cvss v3 score and the
        exploitability subscore from the given comment dictionary.
    """
    exploitability_dict = {}
    impact_dict = {}
    scope_changed = comment_dictionary['Scope']
    exploitability_dict['scope_changed'] = scope_changed
    for key in ['Attack Vector', 'Attack Complexity',
                'Privileges Required', 'User Interaction']:
        exploitability_key = key.lower().replace(' ', '_')
        exploitability_dict[exploitability_key] = comment_dictionary[key]
    for key in ['Confidentiality', 'Integrity', 'Availability']:
        impact_dict[key.lower()] = comment_dictionary[key]
    score = v3_calculator.compute_base_score_from_dicts(
        exploitability_dict, impact_dict, scope_changed)
    exploitability_sub_score = v3_calculator.get_exploitability_sub_score(
        **exploitability_dict)
    return score, round(exploitability_sub_score, 2)



#!/usr/bin/python
import unittest

from . import calculator


class TestCVSS3Calculator(unittest.TestCase):

    def test_score_example_4_4(self):
        """ tests calculating the cvss score of example 4.4
            (base score 9.9) from https://www.first.org/cvss/examples.
        """
        scope = 'Changed'
        exploitability_sub_score = calculator.get_exploitability_sub_score(
            'network',
            'low',
            'low',
            'none',
            scope,
        )
        isc_base = calculator.get_impact_sub_score_base(
            'high',
            'high',
            'high')
        impact_sub_score = calculator.get_impact_sub_score(isc_base, scope)
        base_score = calculator.compute_base_score(
            impact_sub_score, exploitability_sub_score, scope)
        self.assertEqual('%s' % base_score, '9.9')

    def test_score_example_3_4(self):
        """ tests calculating the cvss score of example 3.4
            (base score 3.1) from https://www.first.org/cvss/examples.
        """
        scope = 'Unchanged'
        exploitability_dict = {
            'attack_vector': 'network',
            'attack_complexity': 'high',
            'privileges_required': 'none',
            'user_interaction': 'required',
        }
        impact_dict = {
            'confidentiality': 'Low',
            'integrity': 'none',
            'availability': 'none',
        }
        base_score = calculator.compute_base_score_from_dicts(
            exploitability_dict, impact_dict, scope)
        self.assertEqual('%s' % base_score, '3.1')

    def test_score_low_severity_example(self):
        """ tests calculating a low severity cvss v3 issue
            (base score 1.8)
        """
        scope = 'Changed'
        exploitability_dict = {
            'attack_vector': 'physical',
            'attack_complexity': 'high',
            'privileges_required': 'high',
            'user_interaction': 'required',
        }
        impact_dict = {
            'confidentiality': 'Low',
            'integrity': 'none',
            'availability': 'none',
        }
        base_score = calculator.compute_base_score_from_dicts(
            exploitability_dict, impact_dict, scope)
        self.assertEqual('%s' % base_score, '1.8')

    def test_score_high_privileges_required_unchanged_scope(self):
        """ tests calculating a low severity cvss v3 issue
            (base score 1.8)
        """
        scope = 'Unchanged'
        exploitability_dict = {
            'attack_vector': 'network',
            'attack_complexity': 'low',
            'privileges_required': 'high',
            'user_interaction': 'none',
        }
        impact_dict = {
            'confidentiality': 'Low',
            'integrity': 'Low',
            'availability': 'Low',
        }
        base_score = calculator.compute_base_score_from_dicts(
            exploitability_dict, impact_dict, scope)
        self.assertEqual('%s' % base_score, '4.7')

    def test_get_severity_description_none(self):
        """ tests that get_severity_description works as expected with
            the none rating.
        """
        self.assertEqual(
            calculator.get_severity_description(0.0), 'None')

    def test_get_severity_description_low(self):
        """ tests that get_severity_description works as expected with
            the low rating.
        """
        for value in [0.1, 3.9]:
            self.assertEqual(
                calculator.get_severity_description(value), 'Low')

    def test_get_severity_description_medium(self):
        """ tests that get_severity_description works as expected with
            the medium rating.
        """
        for value in [4.0, 6.9]:
            self.assertEqual(
                calculator.get_severity_description(value), 'Medium')

    def test_get_severity_description_high(self):
        """ tests that get_severity_description works as expected with
            the high rating.
        """
        for value in [7.0, 8.9]:
            self.assertEqual(
                calculator.get_severity_description(value), 'High')

    def test_get_severity_description_critical(self):
        """ tests that get_severity_description works as expected with
            the critical rating.
        """
        for value in [9.0, 10.0]:
            self.assertEqual(
                calculator.get_severity_description(value), 'Critical')

    def test_get_attack_vector_value_with_adjacent_keys(self):
        """ tests that the get_attack_vector_value method works as expected
            with the various adjacent vector keys.
        """
        for key in ['adjacent', 'adjacent network']:
            self.assertEqual(calculator.get_attack_vector_value(key), 0.62)

    def test_get_base_vector_string_dict(self):
        """ tests that the get_base_vector_string_dict works as expected. """
        map = calculator.get_base_vector_string_dict()
        for key, expected in (
                ('N', 'Network'),
                ('A', 'Adjacent'),
                ('L', 'Local'),
                ('P', 'Physical')):
            self.assertEqual(map['av'][key], expected)
        for key, expected in (
                ('L', 'Low'),
                ('H', 'High')):
            self.assertEqual(map['ac'][key], expected)
        for key, expected in (
                ('N', 'None'),
                ('L', 'Low'),
                ('H', 'High')):
            self.assertEqual(map['pr'][key], expected)
        for key, expected in (
                ('N', 'None'),
                ('R', 'Required')):
            self.assertEqual(map['ui'][key], expected)
        for key, expected in (
                ('U', 'Unchanged'),
                ('C', 'Changed')):
            self.assertEqual(map['s'][key], expected)
        for impact in ['c', 'i', 'a']:
            for key, expected in (
                    ('N', 'None'),
                    ('L', 'Low'),
                    ('H', 'High')):
                self.assertEqual(map[impact][key], expected)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python
from __future__ import print_function
import argparse

from cvss_util.cvssv2 import calculator


def setup_args():
    parser = argparse.ArgumentParser(description='CVSS Calculator')
    parser.add_argument(
        '-av',
        '--AccessVector',
        dest='AccessVector',
        choices=[
            'local',
            'adjacent',
            'network'],
        required=True)
    parser.add_argument('-ac', '--AccessComplexity', dest='AccessComplexity',
                        choices=['low', 'medium', 'high'], required=True)
    parser.add_argument('-a', '--Authentication', dest='Authentication',
                        choices=['none', 'single', 'multiple'], required=True)
    _impact_choices = ('none', 'partial', 'complete')
    parser.add_argument('-c', '--ConfImpact', dest='ConfImpact',
                        choices=_impact_choices, default='none')
    parser.add_argument('-i', '--IntegImpact', dest='IntegImpact',
                        choices=_impact_choices, default='none')
    parser.add_argument('-ai', '--AvailImpact', dest='AvailImpact',
                        choices=_impact_choices, default='none')
    return parser


def main():
    parser = setup_args()
    args = parser.parse_args()
    impact_s = calculator.get_impact_score(args.ConfImpact,
                                           args.IntegImpact, args.AvailImpact)
    exploit_s = calculator.get_exploitability_score(
        args.AccessVector,
        args.AccessComplexity,
        args.Authentication)
    score = round(calculator.calc_base_score(impact_s, exploit_s), 1)
    severity = calculator.get_severity_description(score)
    print(severity, abs(score))


if __name__ == "__main__":
    main()



#!/usr/bin/python
import unittest

from . import calculator


class TestCVSSCalculator(unittest.TestCase):

    def test_high_score(self):
        """ tests scoring a persistent xss """
        impact = calculator.get_impact_score('partial',
                                             'partial', 'partial')
        exploit = calculator.get_exploitability_score('network',
                                                      'low', 'single')
        score = round(calculator.calc_base_score(impact, exploit), 1)
        severity = calculator.get_severity_description(score)
        self.assertEqual(score, 6.5)
        self.assertEqual(round(impact, 2), 6.44)
        self.assertEqual(exploit, 7.952)
        self.assertEqual(severity, 'High')

    def test_critical_score(self):
        """ tests scoring a critical issue. """
        exploit = calculator.get_exploitability_score(
            'network',
            'low',
            'single',
        )
        impact = calculator.get_impact_score(
            'Complete',
            'Complete',
            'Complete',
        )
        score = round(calculator.calc_base_score(impact, exploit), 1)
        severity = calculator.get_severity_description(score)
        self.assertEqual(score, 9)
        self.assertEqual(severity, 'Critical')

    def test_example_5_3_score(self):
        """ tests calculating the cvss score of example 5.3
            (base score 4.6) from https://www.first.org/cvss/examples.
        """
        exploit = calculator.get_exploitability_score(
            'local',
            'low',
            'none',
        )
        impact = calculator.get_impact_score(
            'partial',
            'partial',
            'partial',
        )
        score = round(calculator.calc_base_score(impact, exploit), 1)
        severity = calculator.get_severity_description(score)
        self.assertEqual(score, 4.6)
        self.assertEqual(severity, 'Medium')

    def test_low_score(self):
        """ tests calculating the cvss score of a low severity issue. """
        exploit = calculator.get_exploitability_score(
            'local',
            'low',
            'Single',
        )
        impact = calculator.get_impact_score(
            'partial',
            'none',
            'none',
        )
        score = round(calculator.calc_base_score(impact, exploit), 1)
        severity = calculator.get_severity_description(score)
        self.assertEqual(score, 1.7)
        self.assertEqual(severity, 'Low')

    def test_get_severity_description_low(self):
        """ tests that get_severity_description works as expected with
            the low rating.
        """
        for value in [0.0, 2.9]:
            self.assertEqual(
                calculator.get_severity_description(value), 'Low')

    def test_get_severity_description_medium(self):
        """ tests that get_severity_description works as expected with
            the medium rating.
        """
        for value in [3.0, 5.9]:
            self.assertEqual(
                calculator.get_severity_description(value), 'Medium')

    def test_get_severity_description_high(self):
        """ tests that get_severity_description works as expected with
            the high rating.
        """
        for value in [6.0, 7.9]:
            self.assertEqual(
                calculator.get_severity_description(value), 'High')

    def test_get_severity_description_critical(self):
        """ tests that get_severity_description works as expected with
            the critical rating.
        """
        for value in [8.0, 10.0]:
            self.assertEqual(
                calculator.get_severity_description(value), 'Critical')

    def test_get_base_vector_string_dict(self):
        """ tests that the get_base_vector_string_dict works as expected. """
        map = calculator.get_base_vector_string_dict()
        for key, expected in (
                ('L', 'Local'),
                ('A', 'Adjacent Network'),
                ('N', 'Network')):
            self.assertEqual(map['av'][key], expected)
        for key, expected in (
                ('H', 'High'),
                ('M', 'Medium'),
                ('L', 'Low')):
            self.assertEqual(map['ac'][key], expected)
        for key, expected in (
                ('M', 'Multiple'),
                ('S', 'Single'),
                ('N', 'None')):
            self.assertEqual(map['au'][key], expected)
        for impact in ['c', 'i', 'a']:
            for key, expected in (
                    ('N', 'None'),
                    ('P', 'Partial'),
                    ('C', 'Complete')):
                self.assertEqual(map[impact][key], expected)


if __name__ == "__main__":
    unittest.main()
